@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: gbpserial.c,v 1.38 2007-02-05 12:30:38 rousseau Exp $
+ * $Id: gbpserial.c,v 1.39 2008-10-15 16:13:13 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -197,7 +197,7 @@ RESPONSECODE WriteGBP(DWORD lun, DWORD length, PUCHAR buffer)
 RESPONSECODE ReadGBP(DWORD lun, PDWORD length, PUCHAR buffer)
 {
 	int rv, len, to_read, already_read;
-	int i, pcb, edc;
+	int i, pcb, edc, hdr_processed;
 	DWORD buffer_size;
 	int reader = LunToReaderIndex(lun);
 	UCHAR *gbpbuffer = GbpBuffer[reader];
@@ -214,72 +214,12 @@ RESPONSECODE ReadGBP(DWORD lun, PDWORD length, PUCHAR buffer)
 	/* error by default */
 	*length = 0;
 
-	/* use select() to, eventually, timeout */
-	FD_ZERO(&fdset);
-	FD_SET(fd, &fdset);
-	t.tv_sec = SERIAL_TIMEOUT;
-	t.tv_usec = 0;
-
-	i = select(fd+1, &fdset, NULL, NULL, &t);
-	if (i == -1)
-	{
-		DEBUG_CRITICAL2("select: %s", strerror(errno));
-		return STATUS_UNSUCCESSFUL;
-	}
-	else
-		if (i == 0)
-		{
-			DEBUG_COMM2("Timeout! (%d sec)", SERIAL_TIMEOUT);
-			return STATUS_UNSUCCESSFUL;
-		}
-
-	rv = read(fd, gbpbuffer, len);
-	if (rv < 0)
-		return STATUS_UNSUCCESSFUL;
-
-	/* too short */
-	if (rv < 4)
-	{
-		DEBUG_XXD(debug_header, gbpbuffer, rv);
-		ExplainGBP(gbpbuffer, rv);
-		DEBUG_COMM2("only %d byte(s) read", rv);
-
-		return STATUS_UNSUCCESSFUL;
-	}
-
-	/* NAD byte */
-	if ((gbpbuffer[NAD] != NAD_IFD2HOST) && (gbpbuffer[NAD] != NAD_XXX2HOST))
-	{
-		DEBUG_XXD(debug_header, gbpbuffer, rv);
-		ExplainGBP(gbpbuffer, rv);
-		DEBUG_COMM2("wrong NAD byte 0x%02X", gbpbuffer[NAD]);
-
-		return STATUS_NAD_UNKNOWN;
-	}
-
-	/* PCB byte */
-	pcb = gbpbuffer[PCB];
-
-	if (pcb & 0xA0)
-	{
-		int ret;
-
-		DEBUG_XXD(debug_header, gbpbuffer, rv);
-		ret = ExplainGBP(gbpbuffer, rv);
-		DEBUG_COMM("PCB error");
-
-		return ret;
-	}
-
-	/* LEN byte (+4 bytes of protocol) */
-	to_read = gbpbuffer[LEN] + 4;
-
-	/* Nb of bytes already read */
-	already_read = rv;
-
-	/* Nb of bytes until the end of the buffer */
-	len -= rv;
+	/* We need at least 4 bytes */
+	to_read = 4;
+	already_read = 0;
+	hdr_processed = FALSE;
 
+	/* Read loop */
 	while (already_read < to_read)
 	{
 		/* use select() to, eventually, timeout */
@@ -312,6 +252,40 @@ RESPONSECODE ReadGBP(DWORD lun, PDWORD length, PUCHAR buffer)
 
 		already_read += rv;
 		len -= rv;
+
+		/* Process header if we just encountered it */
+		if ((already_read >= 4) && !hdr_processed) {
+			/* NAD byte */
+			if ((gbpbuffer[NAD] != NAD_IFD2HOST)
+				&& (gbpbuffer[NAD] != NAD_XXX2HOST))
+			{
+				DEBUG_XXD(debug_header, gbpbuffer, rv);
+				ExplainGBP(gbpbuffer, rv);
+				DEBUG_COMM2("wrong NAD byte 0x%02X", gbpbuffer[NAD]);
+
+				return STATUS_NAD_UNKNOWN;
+			}
+
+			/* PCB byte */
+			pcb = gbpbuffer[PCB];
+
+			if (pcb & 0xA0)
+			{
+				int ret;
+
+				DEBUG_XXD(debug_header, gbpbuffer, rv);
+				ret = ExplainGBP(gbpbuffer, rv);
+				DEBUG_COMM("PCB error");
+
+				return ret;
+			}
+
+			/* LEN byte (+4 bytes of protocol) */
+			to_read = gbpbuffer[LEN] + 4;
+
+			/* Don't process header anymore */
+			hdr_processed = TRUE;
+		}
 	}
 
 	/* calculate checksum */
