@@ -1,6 +1,6 @@
 /*
  *  GCCmds.c
- *  $Id: GCCmds.c,v 1.39 2007-08-14 16:04:26 rousseau Exp $
+ *  $Id: GCCmds.c,v 1.41 2009-04-12 08:48:23 rousseau Exp $
  *  ifd-GemPC
  *
  *  Created by Jean-Luc Giraud on Sat Oct 20 2001.
@@ -280,7 +280,7 @@ ifd_t GCCmdPowerUp(const DWORD lun, PDWORD nlength, UCHAR buffer[])
 				GCGemCoreError(gc_rv, __FILE__, __LINE__, __FUNCTION__);
 
 				/* new power up */
-				rv = GCMakeCommand(lun, sizeof(cmd_emv), cmd_emv, nlength,
+				rv = GCMakeCommand(lun, sizeof(cmd_no_pps), cmd_no_pps, nlength,
 					buffer, &gc_rv);
 				GCGemCoreError(gc_rv, __FILE__, __LINE__, __FUNCTION__);
 			}
@@ -467,7 +467,7 @@ ifd_t GCMakeCommand(const DWORD lun, const DWORD nLengthIn,
 
 UCHAR GCGemCoreError(const UCHAR rv, const char *file, const int line, const char *function)
 {
-	char *text = NULL;
+	const char *text = NULL;
 	char log_level = PCSC_LOG_ERROR;
 
 	/* what is the GemCore status? */
